package explorviz.live_trace_processing.main;

import java.util.Queue;
import java.util.concurrent.TimeUnit;

import explorviz.live_trace_processing.Constants;
import explorviz.live_trace_processing.configuration.Configuration;
import explorviz.live_trace_processing.configuration.ConfigurationFactory;
import explorviz.live_trace_processing.filter.PipesMerger;
import explorviz.live_trace_processing.filter.SinglePipeConnector;
import explorviz.live_trace_processing.filter.reconstruction.TraceReconstructionFilter;
import explorviz.live_trace_processing.filter.reduction.TracesSummarizationFilter;
import explorviz.live_trace_processing.reader.TCPReader;
import explorviz.live_trace_processing.record.IRecord;

public class FilterConfiguration {
	public static void configureAndStartFilters(final Configuration configuration,
			final Queue<IRecord> sink) {
		final SinglePipeConnector<IRecord> traceReductionConnector = new SinglePipeConnector<IRecord>(
				Constants.TRACE_RECONSTRUCTION_DISRUPTOR_SIZE);

		new TracesSummarizationFilter(traceReductionConnector, TimeUnit.SECONDS.toNanos(1), sink)
		.start();

		final PipesMerger<IRecord> traceReconstructionMerger = new PipesMerger<IRecord>(
				Constants.TCP_READER_DISRUPTOR_SIZE);

		new TraceReconstructionFilter(traceReconstructionMerger,
				traceReductionConnector.registerProducer(),
				Constants.TRACE_RECONSTRUCTION_TIMEOUT_IN_SEC).start();

		new TCPReader(configuration.getIntProperty(ConfigurationFactory.READER_LISTENING_PORT,
				10133), traceReconstructionMerger).read();
	}
}
